#include "constants.h"

/* A pleasant light yellow background color, similar to commercial sticky notes, with black text. */
const guint default_pad_width = 300;
const guint default_pad_height = 300;
const gchar *default_font_name = "Sans 9";
const GdkRGBA default_text_color = {0, 0, 0, 1};
const GdkRGBA default_back_color = {1, 0.933334350586, 0.6, 1};
