#ifndef CONSTANTS_H
#define CONSTANTS_H

#include <gdk/gdk.h>

extern const guint default_pad_width;
extern const guint default_pad_height;
extern const gchar *default_font_name;
extern const GdkRGBA default_text_color;
extern const GdkRGBA default_back_color;

#endif
