/*

Copyright (c) 2002 Jamis Buck
Copyright (c) 2003-2007 Michael Terry
Copyright (c) 2013-2024 Arthur Borsboom

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef __XPAD_TRAY_H__
#define __XPAD_TRAY_H__

#include "xpad-settings.h"

G_BEGIN_DECLS

void xpad_tray_init (XpadSettings *settings);
void xpad_tray_dispose (XpadSettings *settings);
gboolean xpad_tray_has_indicator ();

G_END_DECLS

#endif /* __TRAY_H__ */
